from ether import crypto

from typing import Any, Dict, List
from ether.ether_types import EthABI


def make_type_list(f: Dict[str, Any]) -> str:
    '''
    makes a comma-delimted type list
    for use in the signature and encoding
    e.g. '(bytes,bytes,bytes)'
    '''
    inputs: List[Dict[str, Any]] = f['inputs']
    types = ','.join(t['type'] for t in inputs)
    return f'({types})'


def find(name: str, interface: EthABI) -> List[Dict[str, Any]]:
    return [e for e in interface if 'name' in e and e['name'] == name]


def make_signature(f: Dict[str, Any]) -> str:
    '''
    Parses a function or event ABI into a signture

    Args:
        f (dict): the function or event ABI
    Returns:
        (str): the signature
    '''
    types = make_type_list(f)
    return f'{f["name"]}{types}'


def make_selector(f: Dict[str, Any]) -> bytes:
    '''
    Parses a function ABI into a function selector
    This is the first 4 bytes of the hash of the signature

    Args:
        f (dict): the function ABI
    Returns:
        (bytes): the function selector
    '''
    return crypto.keccak256(make_signature(f).encode('utf8'))[:4]



from Cryptodome.Hash import keccak

from eth_keys.backends.native import ecdsa


def keccak256(msg: bytes) -> bytes:
    '''
    Does solidity's dumb keccak

    Args:
        msg (bytes): the message to hash
    Returns:
        (bytes): the keccak256 digest
    '''
    keccak_hash = keccak.new(digest_bits=256)
    keccak_hash.update(msg)
    return keccak_hash.digest()


def pow_mod(x: int, y: int, z: int) -> int:
    '''
    int, int, int (or float)
    returns (x^y)mod z
    '''
    number = 1
    while y:
        if y & 1:
            number = number * x % z
        y >>= 1
        x = x * x % z
    return number


def uncompress_pubkey(pubkey: bytes) -> bytes:
    '''
    takes a compressed pubkey, returns the uncompressed pubkey (64 bytes)
    '''
    p = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f
    parity = pubkey[0] - 2
    x = int.from_bytes(pubkey[1:], 'big')
    a = (pow_mod(x, 3, p) + 7) % p
    y = pow_mod(a, (p + 1) // 4, p)
    if y % 2 != parity:
        y = -y % p
    return (x.to_bytes(32, 'big')) + (y.to_bytes(32, 'big'))


def compress_pubkey(pubkey: bytes) -> bytes:
    parity = (pubkey[-1] & 1) + 2
    compressed = bytes([parity]) + pubkey[:32]
    return compressed


def priv_to_pub(privkey: bytes) -> bytes:
    return ecdsa.private_key_to_public_key(privkey)


def pub_to_addr(pubkey: bytes) -> str:
    '''Eth addr is last 20 bytes of keccak256 of pubkey'''
    return '0x' + keccak256(pubkey)[-20:].hex()


def priv_to_addr(privkey: bytes) -> str:
    '''Make address from privkey'''
    return pub_to_addr(priv_to_pub(privkey))



import eth_abi

from ether import abi, crypto

from typing import Any, cast, Dict, List
from ether.ether_types import EthABI, ParsedEtherEvent, UnparsedEtherEvent


def _zero_pad_address(addr: str):
    '''Pads an address with 0s for event topics'''
    return '0x' + '00' * 12 + addr[2:]


def _make_topic0(event: Dict[str, Any]) -> str:
    '''
    Calculates the event topic hash frrom the event ABI
    Args:
        event (dict): the event ABI
    Returns:
        (str): the event topic as 0x prepended hex
    '''
    signature = abi.make_signature(event).encode('utf8')
    topic_hex = crypto.keccak256(signature).hex()
    return '0x{}'.format(topic_hex)


def _match_topic0_to_event(
        event_topic: str,
        events: List[Dict[str, Any]]) -> Dict[str, Any]:
    '''
    Finds the corresponding event from a topic string
    Args:
        event_topic (str): the event's 0x prepended hex topic
    Returns:
        (dict): the event ABI
    '''
    for event in events:
        if _make_topic0(event) == event_topic:
            return event
    raise ValueError('Topic not found')


def _find_indexed(event: Dict[str, Any]) -> List[Dict[str, Any]]:
    '''
    Finds indexed arguments
    Args:
        event_topic (str): the event's 0x prepended hex topic
    Returns:
        (list): the indexed arguments
    '''
    return [t for t in event['inputs'] if t['indexed']]


def _find_unindexed(event: Dict[str, Any]) -> List[Dict[str, Any]]:
    '''
    Finds indexed arguments
    Args:
        event_topic (str): the event's 0x prepended hex topic
    Returns:
        (list): the unindexed arguments
    '''
    return [t for t in event['inputs'] if not t['indexed']]


def _process_value(t: str, v: str):
    '''
    Args:
        t (str): the type annotation
        v (str): the value string
    Returns:
        (t): the parsed value
    '''
    # strip prefix if necessary
    if '0x' in v:
        v = v[2:]
    if t == 'address':
        # last 20 bytes of value
        return '0x{}'.format(v[-40:])
    if 'bytes' in t:
        return bytes.fromhex(v)
    if 'uint' in t:
        return int.from_bytes(bytes.fromhex(v), 'big', signed=False)
    elif 'int' in t:
        return int.from_bytes(bytes.fromhex(v), 'big', signed=True)
    elif t == 'bool':
        return t[-1] == '1'


def decode_event(
        encoded_event: UnparsedEtherEvent,
        abi: EthABI) -> Dict[str, Any]:
    '''
    Decodes an event using the provide abi
    Args:
        encoded_event (dict): the etherscan/full node event Dict
        abi           (dict): the abi as a dict (use json.loads)
    Returns:
        (dict): the decoded dict
    '''
    ret = {}

    events = [entry for entry in abi if entry['type'] == 'event']

    # find the abi
    event_abi = _match_topic0_to_event(encoded_event['topics'][0], events)

    # get the indexed args
    indexed = _find_indexed(event_abi)
    for i in range(len(indexed)):
        signature = indexed[i]
        val = _process_value(signature['type'], encoded_event['topics'][i + 1])
        ret[signature['name']] = val

    unindexed = _find_unindexed(event_abi)
    unindexed_values = eth_abi.decode_abi(
        [t['type'] for t in unindexed],
        bytes.fromhex(encoded_event['data'][2:]))

    for k, v in zip([t['name'] for t in unindexed], unindexed_values):
        ret[k] = v

    ret['event_name'] = event_abi['name']

    return ret


def parse_event_data(
        encoded_event: UnparsedEtherEvent,
        contract_abi: EthABI) -> ParsedEtherEvent:
    '''Parses an event given a contract ABI'''
    tmp = cast(ParsedEtherEvent, encoded_event.copy())
    tmp['data'] = decode_event(encoded_event, contract_abi)
    return tmp



import asyncio

from asyncio import Future
from mypy_extensions import TypedDict
from typing import Any, Dict, List, Optional, Tuple

EthSig = Tuple[int, int, int]

EthABI = List[Dict[str, Any]]


class EthTx(TypedDict):
    to: str
    value: int
    gas: int
    gasPrice: int
    nonce: int
    data: bytes


class CeloMod(TypedDict):
    gasCurrency: Optional[str]
    gasFeeRecipient: Optional[str]


class CeloTx(EthTx, CeloMod):
    ...


class SignedEthTx(EthTx):
    v: int
    s: int
    r: int


class SignedCeloTx(SignedEthTx, CeloMod):
    ...


class UnsignedEthTx(EthTx):
    chainId: int


class UnsignedCeloTx(UnsignedEthTx, CeloMod):
    ...


class EtherEvent(TypedDict):
    address: str
    blockHash: str
    blockNumber: str
    logIndex: str
    removed: bool
    topics: List[str]
    transactionHash: str
    transactionIndex: str


class UnparsedEtherEvent(EtherEvent):
    data: str


class ParsedEtherEvent(EtherEvent):
    data: Dict[str, Any]


# we declare it in this style because "from" is a keyword
Receipt = TypedDict(
    'Receipt',
    {
        'blockHash': str,
        'blockNumber': str,
        'contractAddress': Optional[str],
        'cumulativeGasUsed': str,
        'from': str,
        'gasUsed': str,
        'logs': List[EtherEvent],
        'logsBloom': str,
        'status': str,
        'to': str,
        'transactionHash': str,
        'transactionIndex': str

    }
)


class RPCRequest(TypedDict):
    method: str
    params: List
    fut: Future


class RPCSubscription(TypedDict):
    method: str
    params: List
    queue: asyncio.Queue



from eth_utils import to_checksum_address
from eth_account import Account
from eth_account.internal import transactions

from ether.ether_types import UnsignedEthTx, SignedEthTx, EthSig

from typing import cast, Union


def get_signature(tx: UnsignedEthTx, key: bytes) -> EthSig:
    '''
    Gets a signature for an ethereum transaction under a specified key
    '''
    if not tx['chainId']:
        raise ValueError('This library enforces EIP-155.')

    tmp_tx = tx.copy()
    tmp_tx['to'] = to_checksum_address(tx['to'])
    t = Account.signTransaction(tmp_tx, key)

    return t['v'], t['r'], t['s']


def sign_transaction(
        tx: UnsignedEthTx,
        key: bytes) -> SignedEthTx:
    '''
    Signs an ethereum transaction with a specified key
    '''
    sig = get_signature(tx, key)

    return SignedEthTx(
        to=tx['to'],
        value=tx['value'],
        gas=tx['gas'],
        gasPrice=tx['gasPrice'],
        nonce=tx['nonce'],
        data=tx['data'],
        v=sig[0], r=sig[1], s=sig[2])


def serialize(tx: SignedEthTx) -> str:
    '''
    serialize a signed Ethereum transaction
    '''
    temp_tx = cast(dict, tx.copy())

    # NB: serializer wants bytes ¯\_(ツ)_/¯
    #     strip prefix and convert to bytes
    if 'x' in temp_tx['to']:
        temp_tx['to'] = bytes.fromhex(temp_tx['to'][2:])
    else:
        temp_tx['to'] = bytes.fromhex(temp_tx['to'])

    temp_tx_obj = transactions.Transaction.from_dict(temp_tx)

    v = temp_tx.pop('v')
    r = temp_tx.pop('r')
    s = temp_tx.pop('s')

    # NB: the serializer wants the signature passed through
    #     both inside AND outside the object ಠ_ಠ
    return transactions.encode_transaction(temp_tx_obj, (v, r, s)).hex()


def recover_sender(tx: Union[SignedEthTx, str]) -> str:
    '''
    Recover the sender from a signed ethereum transaction
    '''
    if type(tx) == dict:
        t = serialize(cast(SignedEthTx, tx))
    else:
        t = cast(str, tx)

    return Account.recoverTransaction(t)



import abc
import ssl
import json
import aiohttp
import asyncio
import websockets

from ether import transactions

from asyncio import Future
from logging import Logger
from websockets.client import WebSocketClientProtocol
from ether.ether_types import EthTx, Receipt, RPCRequest
from ether.ether_types import RPCSubscription, SignedEthTx, UnparsedEtherEvent
from typing import Any, cast, Dict, Generator, List, Optional, Tuple, Union


def _id(start: int) -> Generator[int, None, None]:
    '''Infinite generator for unique request IDs'''
    index = start
    while True:
        yield index
        index += 1


def get_client(
        network: str,
        **kwargs) -> 'BaseRPC':
    '''
    Convenience method for getting a client.
    Prefer instantiating directly

    Args:
        network (str): mainnet, ropsten, rinkeby, etc.
        force_https (bool): set true to disable websockets
    '''
    use_infura = 'infura_key' in kwargs \
                 and kwargs['infura_key'] is not None \
                 and len(kwargs['infura_key']) > 0
    force_https = 'force_https' in kwargs and kwargs['force_https']
    if use_infura:
        return InfuraWSRPC(network=network, **kwargs)
    if use_infura and force_https:
        return InfuraHTTPRPC(network=network, **kwargs)

    uri = ''
    if 'uri' in kwargs:
        uri = kwargs.pop('uri')
    else:
        raise ValueError('Must specify URI or infura key')
    if uri[0:3] == 'wss' and force_https:
        return WSRPC(uri, network, **kwargs)
    return HTTPRPC(uri, network, **kwargs)


class BaseRPC(metaclass=abc.ABCMeta):

    _ids: Generator[int, None, None]
    uri: str
    network: str
    connected: bool

    params: Dict[str, Any]

    def __repr__(self) -> str:
        return f'Eth RPC {self.mode} client: {self.network} via {self.uri}'

    def __init__(
            self,
            uri: str,
            network: str,
            logger: Optional[Logger] = None,
            **kwargs) -> None:

        self._ids = _id(kwargs['_id'] if '_id' in kwargs else 0)
        self.connected = False

        self.uri = uri
        self.network = network
        self.mode = uri[0:3]
        self.params = kwargs

        self._logger = logger

    def info(self, *args, **kwargs) -> None:
        if self._logger is not None:
            self._logger.info(*args, **kwargs)

    def debug(self, *args, **kwargs) -> None:
        if self._logger is not None:
            self._logger.debug(*args, **kwargs)

    def warn(self, *args, **kwargs) -> None:
        if self._logger is not None:
            self._logger.warn(*args, **kwargs)

    def error(self, *args, **kwargs) -> None:
        if self._logger is not None:
            self._logger.error(*args, **kwargs)

    @abc.abstractmethod
    async def open(self):
        ...

    @abc.abstractmethod
    async def close(self):
        ...

    @abc.abstractmethod
    async def _RPC(self, method: str, params: List):
        ...

    @staticmethod
    def _encode_int(v: Any) -> Any:
        if isinstance(v, int):
            return hex(cast(int, v))
        return v

    @staticmethod
    def _shallow_prep_params(params: List[Any]) -> List[Any]:
        return list(map(BaseRPC._encode_int, params))

    async def send_transaction(
            self,
            from_addr: str,
            tx: EthTx) -> str:
        '''Send a transaction, let the node sign it if it wants'''
        if hasattr(self, 'infura_key'):
            raise RuntimeError('Tried to sign tx with infura connection')

        param_obj = cast(dict, tx.copy())
        param_obj['from'] = from_addr

        if 'data' in param_obj:
            param_obj['data'] = f'0x{param_obj["data"].hex()}'

        if 'chainId' in param_obj:
            param_obj.pop('chainId')

        for key, value in param_obj.items():
            param_obj[key] = BaseRPC._encode_int(value)

        res = await self._RPC(
            method='eth_sendTransaction',
            params=[param_obj]
        )

        return cast(str, res)

    async def get_balance(
            self,
            address: str,
            block: Union[str, int] = 'latest') -> int:
        '''Gets the number of wei at an address'''
        res = await self._RPC(
            'eth_getBalance',
            [address, BaseRPC._encode_int(block)])
        return int(res, 16)

    async def get_logs(
            self,
            address: Optional[str] = None,
            from_block: Union[str, int] = 'earliest',
            to_block: Union[str, int] = 'latest',
            topics: Optional[List[Optional[str]]] = None,
            blockhash: Optional[str] = None) -> List[UnparsedEtherEvent]:
        '''Gets logs'''
        params: Dict[str, Union[str, List[Optional[str]]]] = {}

        if address:
            params['address'] = address
        if topics:
            params['topics'] = topics

        # can't have both a hash and a height range
        if blockhash:
            params['blockhash'] = blockhash
        else:
            params['fromBlock'] = BaseRPC._encode_int(from_block)
            params['toBlock'] = BaseRPC._encode_int(to_block)

        return await self._RPC(method='eth_getLogs', params=[params])

    async def get_past_contract_logs(
            self,
            address: str,
            topics: Optional[List[Optional[str]]]) -> List[UnparsedEtherEvent]:
        '''Simpler method to get contract logs'''
        return await self.get_logs(address=address, topics=topics)

    async def broadcast(self, tx: str) -> str:

        '''Broadcast a transaction to the network'''
        if tx[0:2] != '0x':
            tx = f'0x{tx}'
        return await self._RPC(
            method='eth_sendRawTransaction',
            params=[tx])

    async def get_tx_receipt(
            self,
            tx_id: str) -> Optional[Receipt]:
        '''Get the receipt of a transaction. None if not confirmed'''
        if tx_id[:2] != '0x':
            tx_id = f'0x{tx_id}'

        return await self._RPC(
            method='eth_getTransactionReceipt',
            params=[tx_id])

    async def get_nonce(self, account: str) -> int:
        res = await self._RPC(
            method='eth_getTransactionCount',
            params=[account, 'pending'])
        appropriate_nonce = int(res, 16)
        return appropriate_nonce

    async def preflight_tx(
            self,
            tx: EthTx,
            sender: Optional[str] = None):
        '''Preflight a transaction'''
        if sender is None and 'v' in tx:
            sender = transactions.recover_sender(cast(SignedEthTx, tx))
        else:
            sender = '0x' + '11' * 20

        res = await self._RPC(
            method='eth_call',
            params=[
                {
                    'from': sender,
                    'to': tx['to'],
                    'data': f'0x{tx["data"].hex()}'
                },
                'latest'  # block height parameter
            ])
        return res


class WSRPC(BaseRPC):

    _ws: WebSocketClientProtocol

    _inflight: Dict[int, RPCRequest] = {}  # requests awaiting responses
    _subscriptions: Dict[str, RPCSubscription] = {}  # subscription queues

    _ping_task: Future
    _handle_task: Future

    def __init__(
            self,
            uri: str,
            network: str,
            **kwargs) -> None:
        super().__init__(uri, network, **kwargs)

    def __repr__(self):
        '''Override repr to add connection status'''
        return f'{super().__repr__()}\tConnected: {self.connected}'

    async def open(self):
        self._ws = await websockets.connect(self.uri, ssl=ssl.SSLContext())
        self.connected = True
        self._ping_task = asyncio.ensure_future(self._ping())
        self._handle_task = asyncio.ensure_future(self._handle_incoming())

    async def _handle_incoming(self) -> None:
        '''Router for incoming messages. Distributes them to futures/queues'''
        try:
            while True:
                msg = await self._ws.recv()
                payload = json.loads(msg)
                # if it has a request ID, it's an RPC response
                if 'error' in payload:
                    self._inflight[payload['id']]['fut'].set_exception(
                        RuntimeError(str(payload['error'])))
                    continue
                elif 'id' in payload and payload['id'] in self._inflight:
                    future = self._inflight[payload['id']]['fut']
                    future.set_result(payload['result'])
                # otherwise it's a subscript notification
                # add it to the subscription's queue
                elif ('params' in payload
                      and 'subscription' in payload['params']):
                    sub_id = payload['params']['subscription']
                    q = self._subscriptions[sub_id]['queue']
                    await q.put(payload['params']['result'])
        except websockets.exceptions.ConnectionClosed:
            if self.connected:
                # if the forloop breaks due to a connection closed error
                await self.close()

        except asyncio.CancelledError:
            pass

    async def close(self) -> None:
        '''Close the socket and cancel associated tasks'''
        if not hasattr(self, '_ws'):
            return

        await self._ws.close()

        if not self.connected:
            return
        self.connected = False
        self._ping_task.cancel()
        self._handle_task.cancel()
        await asyncio.gather(
            self._ping_task,
            self._handle_task,
            return_exceptions=True)

    def get_pending(
            self) -> Tuple[int, List[RPCRequest], List[RPCSubscription]]:
        if self.connected:
            raise RuntimeError(
                'Tried to retrive pending requests from active socket')

        pending: List[RPCRequest] = []
        subs: List[RPCSubscription] = []

        for _, r in self._inflight.items():
            if not r['fut'].done():
                pending.append(r)
        for _, s in self._subscriptions.items():
            subs.append(s)

        # what's a safe ID to resume from to prevent collisions
        next_id = max(self._inflight.keys()) + 1

        return next_id, pending, subs

    @classmethod
    async def from_pending(
            C,
            uri: str,
            network: str,
            start_id: int,
            pending: List[RPCRequest] = [],
            subs: List[RPCSubscription] = [],
            **kwargs) -> 'WSRPC':
        '''
        Returns an open connection, using requests and subscriptions from
        another WSRPC instance. Subscriptions are restarted, and inflight reqs
        are remade. If you pass a pending request from another closed socket,
        the new socket will handle resolving the associated futures.

        Useful for seamlessly(ish) resuming a faulted session
        '''
        _id = start_id
        new = C(uri=uri, network=network, _id=_id, **kwargs)

        await new.open()

        reqs = [new._RPC(r['method'], r['params'], r['fut']) for r in pending]
        asyncio.ensure_future(asyncio.gather(*reqs))

        new_subs = [new._subscribe(s['params'], s['queue']) for s in subs]
        asyncio.ensure_future(asyncio.gather(*new_subs))
        return new

    async def _ping(self):
        try:
            while True:
                await self._ws.ping()
                await asyncio.sleep(15)
        except websockets.exceptions.ConnectionClosed:
            if self.connected:
                await self.close()

    async def _RPC(
            self,
            method: str,
            params: List,
            fut: Optional[Future] = None):
        '''Internal method for handling RPC calls'''
        # get a new ID and the websket
        req_id = next(self._ids)

        # Create a new future store it in our pendings and send it
        if fut is None:
            future = asyncio.get_event_loop().create_future()
        else:
            future = cast(Future, fut)
        self._inflight[req_id] = RPCRequest(
            method=method,
            params=params,
            fut=future)

        # send the request
        payload = {
            'jsonrpc': '2.0',
            'id': req_id,
            'method': method,
            'params': params
        }

        self.debug(f'dispatching ws payload {payload}')
        try:
            await self._ws.send(json.dumps(payload))
        except websockets.exceptions.ConnectionClosed:
            if self.connected:
                await self.close()
            raise RuntimeError('Websocket connection closed unexpectedly')

        # await the result
        res = await future
        return res

    async def _subscribe(
            self,
            params: List,
            queue: Optional[asyncio.Queue] = None) \
            -> Tuple[str, asyncio.Queue]:
        '''Internal method for setting up subscriptions'''
        method = 'eth_subscribe'

        # set up the queue
        q: asyncio.Queue
        if queue is None:
            q = asyncio.Queue()
        else:
            q = cast(asyncio.Queue, queue)

        # get the subscription ID and store the q
        sub_id = await self._RPC(method, params)
        self._subscriptions[sub_id] = RPCSubscription(
            method=method,
            params=params,
            queue=q)
        return sub_id, q

    async def unsubscribe(self, sub_ids: List[str]):
        '''Cancel a list of subscriptions'''

        res = await self._RPC('eth_unsubscribe', sub_ids)
        for sub in sub_ids:
            self._subscriptions.pop(sub)
        return res

    async def subscribe_to_address_events(
            self,
            addresses: List[str],
            topics: Optional[List[str]]) -> Tuple[str, asyncio.Queue]:
        '''Subscribes to event logs at specific addresses'''
        param_dict = {'address': addresses}
        if topics:
            param_dict['topics'] = topics

        return await self._subscribe(['logs', param_dict])

    async def subscribe_to_headers(self) -> Tuple[str, asyncio.Queue]:
        '''Subscribes to new headers on the network'''
        return await self._subscribe(['newHeads'])


class HTTPRPC(BaseRPC):

    _session: aiohttp.ClientSession

    def __init__(
            self,
            uri: str,
            network: str,
            **kwargs) -> None:
        super().__init__(uri, network, **kwargs)

    async def open(self) -> None:
        if self.connected:
            return
        # close the connection after each request
        # some overhead, but less likely to get server-side disconnects
        self._session = aiohttp.ClientSession(
            headers={"Connection": "close"}
        )
        self.connected = True

    async def close(self) -> None:
        await self._session.close()
        self.connected = False

    async def _RPC(self, method: str, params: List[Any]) -> Dict[str, Any]:
        payload = {
            "jsonrpc": "2.0",
            "id": next(self._ids),
            "method": method,
            "params": BaseRPC._shallow_prep_params(params)}
        self.debug(f'dispatching post request {payload}')
        resp = await self._session.post(self.uri, json=payload)
        if resp.status != 200:
            raise RuntimeError(f'Bad status during RPC request: {resp.status}')
        resp_json = await resp.json()
        result = resp_json['result'] if 'result' in resp_json else resp_json
        return result


class InfuraHTTPRPC(HTTPRPC):

    def __init__(self, network: str, infura_key: str, **kwargs) -> None:
        uri = f'https://{network}.infura.io/v3/{infura_key}'
        kwargs['uri'] = uri
        self.infura_key = infura_key
        super().__init__(network=network, infura_key=infura_key, **kwargs)


class InfuraWSRPC(WSRPC):
    def __init__(self, network: str, infura_key: str, **kwargs) -> None:
        uri = f'wss://{network}.infura.io/ws/v3/{infura_key}'
        kwargs['uri'] = uri
        self.infura_key = infura_key
        super().__init__(network=network, infura_key=infura_key, **kwargs)



# flake8: noqa

from setuptools import setup, find_packages

reqs = [
    'eth-abi==1.3.0',
    'eth-keys==0.2.1',
    'eth-account==0.3.0',
    'pycryptodomex',
    'websockets'
]

setup(
    name='riemann-ether',
    version='4.1.4',
    description=('Transaction creation library for Ethereum'),
    url='https://github.com/summa-tx/riemann-ether',
    author='James Prestwich',
    author_email='james@summa.one',
    install_requires=reqs,
    packages=find_packages(),
    package_dir={'ether': 'ether'},
    package_data={'ether': ['py.typed']},
    keywords = 'ethereum cryptocurrency blockchain development',
    python_requires='>=3.6',
    classifiers=[
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)'
    ]
)
